<?php
namespace Teknisa\CADFIS\Controllers;


use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Model\Geral\Entities\Bairro;
use Teknisa\Libs\Controller\CRUD;
use Teknisa\Libs\Util\DateUtil;
//use Teknisa\Geral\Services\BuscarCepCorreiosLogic;

class Ger00200EmpresaController extends TeknisaCrud {

  protected $dataSourceName = "empresa_cadastroEmpresa";
  protected $parameterBagColumns = array("");
  protected $dataSourceNameToSave ='empresa';
  protected $entityManager;
  protected $environment;
  protected $Ger00200EmpresaService;


  public function __construct($crudService, $parameterBag, $environment, $entityManager,$Ger00200EmpresaService) {
      parent::__construct($crudService, $parameterBag);
      $this->environment = $environment;
      $this->entityManager = $entityManager;
      $this->Ger00200EmpresaService = $Ger00200EmpresaService;
  }


  public function salvarInscricaoEstadual(DTO\Request $request, DTO\Response $response) {
    try {
      //$row = $this->crudService->getRowsToSave($request, 'inscrestad');
      $dataSet = $this->Ger00200EmpresaService->criaDataSetInscricaoEstadual(array($request->getRow()));
      $request2 = new DTO\Request\DataSet($dataSet, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
      
      $this->dataSourceNameToSave = 'inscrestad';
      parent::save($request2, $response);
      $this->Ger00200EmpresaService->gravaCertificado($request->getRow());
    } catch (\Exception $e) {
      $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
    }
  }

  public function save(DTO\Request $request, DTO\Response $response) {
  //  $select = $this->Ger00200EmpresaService->selectparamempresa();
    try {
      $this->dataSourceName = 'empresa_cadastroEmpresa';
      $this->dataSourceNameToSave = 'empresa';
      $this->parameterBagColumns = array("");
      $dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
      $rows = $this->crudService->getRowsToSave($request, $dataSourceName);
      $buscar = $this->Ger00200EmpresaService->selectparamempresa($rows[0]['CDEMPRESA']);
      
      $this->crudService->save($dataSourceName, $rows);
      
    } catch (\Exception $e) {
      $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
    }
  }

  public function find(DTO\Request\Filter $request, DTO\Response $response) {
    try {
      $this->dataSourceName = 'empresa_cadastroEmpresa';
      $this->parameterBagColumns = array("");
      $filterCriteria = $request->getFilterCriteria();
      $filterCriteria->setDataSourceName($this->dataSourceName);
      $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
      $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
      $this->crudService->find($response, $newFilterCriteria);
    } catch (\Exception $e) {
      $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
    }
  }

  public function delete(DTO\Request $request, DTO\Response $response) {
    try {
      $this->dataSourceName = 'empresa_cadastroEmpresa';
      $this->parameterBagColumns = array("");
      $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
      $dataSet = $request->getDataSet();
      $dataSet->setDataSourceName($this->dataSourceName);
      $this->crudService->delete($response, $dataSet);
    } catch (\Exception $e) {
      $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
    }
  }

  public function salvarNovaEmpresa(DTO\Request $request, DTO\Response $response) {
    try {
      $this->dataSourceName = 'empresa_cadastroEmpresa';
      $this->dataSourceNameToSave = 'empresa_cadastroEmpresa';
      $this->parameterBagColumns = array("");
      $row = $request->getRow();
      if($row['__is_new']){
        $conditions = $this->condicaoNovaEmpresa($row);
        $resul = $this->realizaConsultaBanco($conditions);
        if(empty($resul->getRows())){
            $dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
            $rows = $this->crudService->getRowsToSave($request, $dataSourceName);
            $this->crudService->save($dataSourceName, $rows);
        }else{
            $response->setError(new DTO\Response\Error("CNPJ já cadastrado.", 1));
        }
      }else{
        $dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
        $rows = $this->crudService->getRowsToSave($request, $dataSourceName);
        $this->crudService->save($dataSourceName, $rows);
      }

    } catch (\Exception $e) {
      $response->setError(new Error("Código já existe", 500));
    }
  }

  public function salvarDadosAdicionais(DTO\Request $request, DTO\Response $response) {
    $select = $this->Ger00200EmpresaService->select();
    if($select == null){
      $this->Ger00200EmpresaService->insert();
    }
        try {
      $this->dataSourceName = 'empresa_cadastroEmpresa';
      $this->dataSourceNameToSave = 'empresa';
      $this->parameterBagColumns = array("");
      $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
            $row = $request->getDataSet()->getRows()[0];
            $dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
            $rows = $this->crudService->getRowsToSave($request, $dataSourceName);
            $this->crudService->save($dataSourceName, $rows);
            $this->entityManager->flush();
      $this->atualizarFilial($row,$response,$request);
      $connection->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

  public function atualizarFilial($row,$response,$request){
      try {
        $conditions = $this->retornaCondicao($row);
        $this->dataSourceName = "filial";
            $this->dataSourceNameToSave ='filial';
        $resul = $this->realizaConsulta($response,$conditions);
        $resul = $resul->getDataSets()[0];
        if(!empty($resul->getRows())){
            $tamanho = count($resul->getRows());
            $array = array();
            for($i=0;$i<$tamanho;$i++){
                $rows = new Row();
            $rows->set('__is_new',  false);
            $rows->set('__createdLocal',true);
            $rows->set('NRORG', $resul->getRows()[$i]['NRORG']);
            $rows->set('CDFILIAL', $resul->getRows()[$i]['CDFILIAL']);
            $rows->set('CDEMPRESA',$resul->getRows()[$i]['CDEMPRESA']);
            $rows->set('IDCUMUPISCOFIL', $row['IDCUMUPICOEMP']);
            $array[] = $rows;
            }
            $dataSet = new DataSet('filial', $array);
            $request2 = new DTO\Request\DataSet($dataSet, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
          $saveRows = $this->crudService->getRowsToSave($request2,$this->dataSourceName);
          $this->crudService->save('filial', $saveRows);
          $this->entityManager->flush();
        }

      }catch (\Exception $e) {
        $this->entityManager->rollback();
        $response->setError(new Error($e->getMessage(), 500));
      }
  }

  public function retornaCondicao($row) {
    $conditions = array(
        array(
            "value" => $row['CDEMPRESA'],
            "operator" => "=",
            "columnName" => "CDEMPRESA"
        )
    );
    return $conditions;
  }

  public function realizaConsulta($response,$conditions) {
      $filterCriteria = new FilterCriteria($this->dataSourceName,$conditions);
      $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
      $this->crudService->find($response,$newFilterCriteria);
      return $response;
  }
    /*public function buscaCep(Row $request, Response $response){
        $rows = $request->getRow();

        $cep = $rows['NRCEPEMPR'];

        $vetEndereco = BuscarCepCorreiosLogic::buscar($cep);

        $response->addDataset(new DataSet('empresa_cadastroEmpresa', array($vetEndereco)));
    }*/
  public function condicaoNovaEmpresa($row) {
      $conditions = array(
        array(
          "value" => $this->environment->getNrOrgTrab(),
          "operator" => "=",
          "columnName" => "NRORG"
        ),
        array(
          "value" => $row['NRINSJUREMPR'],
          "operator" => "=",
          "columnName" => "NRINSJUREMPR"
        )
      );
      return $conditions;
  }

  public function realizaConsultaBanco($conditions) {
      $filterCriteria = new FilterCriteria($this->dataSourceName,$conditions);
      $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
      $dataSet = $this->Ger00200EmpresaService->buscar($newFilterCriteria);
      return $dataSet;
  }

  public function findIncricaoEstadual(DTO\Request\Filter $request, DTO\Response $response) {
    try {
      $this->dataSourceName = 'inscrestad_completa';
      $this->parameterBagColumns = array("P_CDEMPRESA");
      $filterCriteria = $request->getFilterCriteria();
      $filterCriteria->setDataSourceName($this->dataSourceName);
      $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
      $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
      $this->crudService->find($response, $newFilterCriteria);
    } catch (\Exception $e) {
      $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
    }
  }

  public function saveInscricaoEstadual(DTO\Request $request, DTO\Response $response) {
    try {
      $this->dataSourceName = 'inscrestad_completa';
      $this->dataSourceNameToSave = 'inscrestad';
      $dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
      $rows = $this->crudService->getRowsToSave($request, $dataSourceName);
      $this->crudService->save($dataSourceName, $rows);
    } catch (\Exception $e) {
      $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
    }
  }

  public function deleteIncricaoEstadual(DTO\Request $request, DTO\Response $response) {
    try {
      $this->dataSourceName = 'inscrestad_completa';
      $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
      $dataSet = $request->getDataSet();
      $dataSet->setDataSourceName($this->dataSourceName);
      $this->crudService->delete($response, $dataSet);
    } catch (\Exception $e) {
      $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
    }
  }

  public function findIncricaoMunicipal(DTO\Request\Filter $request, DTO\Response $response) {
    try {

      $this->dataSourceName = 'inscrmunicip_completa';
      $this->parameterBagColumns = array("P_CDEMPRESA");
      $filterCriteria = $request->getFilterCriteria();
      $filterCriteria->setDataSourceName($this->dataSourceName);
      $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
      $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
      $this->crudService->find($response, $newFilterCriteria);
    } catch (\Exception $e) {
      $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
    }
  }

  public function saveInscricaoMunicipal(DTO\Request $request, DTO\Response $response) {
    try {
      $this->dataSourceName = 'inscrmunicip_completa';
      $this->dataSourceNameToSave = 'inscrmunicip_completa';
      $dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
      //$dataSet = $request->getDataSet();
      //$rows = $this->crudService->getRowsToSave($request, false);
      $rows = $request->getDataSet()->getRows();
      $this->crudService->save($dataSourceName, $rows);
    } catch (\Exception $e) {
      $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
    }
  }

  public function deleteIncricaoMunicipal(DTO\Request $request, DTO\Response $response) {
    try {
      $this->dataSourceName = 'inscrmunicip_completa';
      $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
      $dataSet = $request->getDataSet();
      $dataSet->setDataSourceName($this->dataSourceName);
      $this->crudService->delete($response, $dataSet);
    } catch (\Exception $e) {
      $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
    }
  }
  
  public function validateNeighborhood(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = 'bairro';
        $this->parameterBagColumns = array("CDPAIS", "SGESTADO", "CDMUNICIPIO");
        $filter = $request->getFilterCriteria()->getConditions();
        
        if($filter){
            $filterToFind = array_filter($filter, function($element){
                if($element['operator'] != 'LIKE_ALL')
                    return $element; 
            });
            $neighborhoodIndex = array_filter($filter, function($element){
                if($element['operator'] == 'LIKE_ALL')
                    return $element; 
            });
            $country = null;
            $state = null;
            $county = null;
            $neighborhood = null;
            
            $i = 0;
            foreach($filterToFind as $adressFilter){
                switch($adressFilter['columnName']){
                    case "CDPAIS": 
                        $country = $adressFilter['value'];
                        break;
                    case "SGESTADO":
                        $state = $adressFilter['value'];
                        break;
                    case "CDMUNICIPIO": 
                        $county = $adressFilter['value'];
                        break;
                    case "NMBAIRRO":
                        $neighborhood = $adressFilter['value'];
                        unset($filterToFind[$i]);
                        break;
                }
                $i++;
            }
            
            if(!empty($neighborhoodIndex) ){
              foreach($neighborhoodIndex as $neighborhoodArray){
                  $neighborhood = $neighborhoodArray['value'];
              }
              $neighborhood = str_replace('%', '', $neighborhood);
            }
            
            $result = $this->crudService->getFilteredDataSet($this->dataSourceName, $filterToFind);
            
            $neighborhoods = $result->getRows();
            
            if(!$neighborhoods) {
                $sequential = '0000000001';
            }
            else {
                
                $largest = 0;
    
                for( $i = 0; $i < count($neighborhoods); $i++){
                    if($largest < $neighborhoods[$i]['CDBAIRRO']) {
                    $largest = $neighborhoods[$i]['CDBAIRRO']; 
                    }
                }
                $sequential = $largest+1;
                $sequential = str_pad($sequential, 10, "0", STR_PAD_LEFT); 
            }
            
            $bairro = $this->Ger00200EmpresaService->getBairroByName($country, $state, $county, $neighborhood);
            if(empty($bairro)){
              $this->insertNewNeighborhood($sequential, $country, $state, $county, $neighborhood);
              $bairro = $this->Ger00200EmpresaService->getBairro($sequential, $country, $state, $county);
            }
        }
        
        $dataSet = new DataSet('bairro', $bairro);
        $response->setDataSets(array($dataSet));
        
    }
    
    public function insertNewNeighborhood($sequential, $country, $state, $county, $neighborhood){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
          
            $nrorg = $this->environment->getNrOrgTrab();
            $bairro = new Bairro();
            $bairro->setCdbairro($sequential);
            $bairro->setCdpais($country);
            $bairro->setSgestado($state);
            $bairro->setCdmunicipio($county);
            $bairro->setNmbairro($neighborhood);
            $bairro->setNrorg($nrorg);
            $bairro->setDtinclusao(DateUtil::getDataAtual());
            $this->entityManager->persist($bairro);
            $this->entityManager->flush();
            $connection->commit();
            return $bairro;
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }


    public function alterarEstrutura(DTO\Request $request, DTO\Response $response) {
        try {
            $operador = $request->getRow();
            $this->alterarEstruturaService($operador['CDFILIAL'], $this->environment->getCdoperador(), $operador['NRORG']);
            $row = array();
            $row['CDFILIAL'] = $operador['CDFILIAL'];
            $row['CDOPERADOR'] = $this->environment->getCdoperador();
            $row['NRORG'] = $operador['NRORG'];
            $dataSet = new DataSet('operador', array($row));
            $response->setDataSets(array($dataSet));
        }catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }


    public function alterarEstruturaService($cdfilial, $cdoperador, $nrorg) {
        $operador = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Operador')
            ->findOneBy(
                array(
                    'cdoperador' => $cdoperador,
                    'nrorg' => $nrorg
                )
            );
        $operador->setNrorgtrab($nrorg);
        $operador->setCdfilialselecionada($cdfilial);
        $operador->setNrorgselecionada($nrorg);
        $this->entityManager->persist($operador);
        $this->entityManager->flush();
        $this->environment->setCdFilial($cdfilial);
    }
    
    public function saveLogoEmpresa(DTO\Request $request, DTO\Response $response) {

        $this->Ger00200EmpresaService->saveLogoEmpresa($request->getRow());
       
    }
}
    